@@ -2,7 +2,8 @@
 
 from sympy.core.numbers import I
 from sympy.polys.polyerrors import CoercionFailed
-from sympy.polys.domains import ZZ, QQ
+from sympy.polys.domains.integerring import ZZ
+from sympy.polys.domains.rationalfield import QQ
 from sympy.polys.domains.algebraicfield import AlgebraicField
 from sympy.polys.domains.domain import Domain
 from sympy.polys.domains.domainelement import DomainElement
@@ -12,20 +13,22 @@ from sympy.polys.domains.ring import Ring
 
 class GaussianElement(DomainElement):
     """Base class for elements of Gaussian type domains."""
-    base = None  # type: Domain
-    _parent = None  # type: Domain
+    base: Domain
+    _parent: Domain
 
     __slots__ = ('x', 'y')
 
-    def __init__(self, x, y=0):
-        conv = self.base.convert
-        self.x = conv(x)
-        self.y = conv(y)
+    def __new__(cls, x, y=0):
+        conv = cls.base.convert
+        return cls.new(conv(x), conv(y))
 
     @classmethod
     def new(cls, x, y):
         """Create a new GaussianElement of the same domain."""
-        return cls(x, y)
+        obj = super().__new__(cls)
+        obj.x = x
+        obj.y = y
+        return obj
 
     def parent(self):
         """The domain that this is an element of (ZZ_I or QQ_I)"""
@@ -270,7 +273,9 @@ class GaussianDomain():
         """Inject generators into this domain. """
         return self.poly_ring(*gens)
 
-    # Override the negative etc handlers because this isn't an ordered domain.
+    def canonical_unit(self, d):
+        unit = self.units[-d.quadrant()]  # - for inverse power
+        return unit
 
     def is_negative(self, element):
         """Returns ``False`` for any ``GaussianElement``. """
@@ -292,10 +297,18 @@ class GaussianDomain():
         """Convert a GMPY mpz to ``self.dtype``."""
         return K1(a)
 
+    def from_ZZ(K1, a, K0):
+        """Convert a ZZ_python element to ``self.dtype``."""
+        return K1(a)
+
     def from_ZZ_python(K1, a, K0):
         """Convert a ZZ_python element to ``self.dtype``."""
         return K1(a)
 
+    def from_QQ(K1, a, K0):
+        """Convert a GMPY mpq to ``self.dtype``."""
+        return K1(a)
+
     def from_QQ_gmpy(K1, a, K0):
         """Convert a GMPY mpq to ``self.dtype``."""
         return K1(a)
@@ -412,9 +425,9 @@ class GaussianIntegerRing(GaussianDomain, Ring):
     """
     dom = ZZ
     dtype = GaussianInteger
-    zero = dtype(0, 0)
-    one = dtype(1, 0)
-    imag_unit = dtype(0, 1)
+    zero = dtype(ZZ(0), ZZ(0))
+    one = dtype(ZZ(1), ZZ(0))
+    imag_unit = dtype(ZZ(0), ZZ(1))
     units = (one, imag_unit, -one, -imag_unit)  # powers of i
 
     rep = 'ZZ_I'
@@ -425,6 +438,24 @@ class GaussianIntegerRing(GaussianDomain, Ring):
     def __init__(self):  # override Domain.__init__
         """For constructing ZZ_I."""
 
+    def __eq__(self, other):
+        """Returns ``True`` if two domains are equivalent. """
+        if isinstance(other, GaussianIntegerRing):
+            return True
+        else:
+            return NotImplemented
+
+    def __hash__(self):
+        """Compute hash code of ``self``. """
+        return hash('ZZ_I')
+
+    @property
+    def has_CharacteristicZero(self):
+        return True
+
+    def characteristic(self):
+        return 0
+
     def get_ring(self):
         """Returns a ring associated with ``self``. """
         return self
@@ -438,7 +469,7 @@ class GaussianIntegerRing(GaussianDomain, Ring):
 
         Also multiply the other arguments by the same power of i.
         """
-        unit = self.units[-d.quadrant()]  # - for inverse power
+        unit = self.canonical_unit(d)
         d *= unit
         args = tuple(a*unit for a in args)
         return (d,) + args if args else d
@@ -583,8 +614,10 @@ class GaussianRationalField(GaussianDomain, Field):
     """
     dom = QQ
     dtype = GaussianRational
-    zero = dtype(0, 0)
-    one = dtype(1, 0)
+    zero = dtype(QQ(0), QQ(0))
+    one = dtype(QQ(1), QQ(0))
+    imag_unit = dtype(QQ(0), QQ(1))
+    units = (one, imag_unit, -one, -imag_unit)  # powers of i
 
     rep = 'QQ_I'
 
@@ -594,6 +627,24 @@ class GaussianRationalField(GaussianDomain, Field):
     def __init__(self):  # override Domain.__init__
         """For constructing QQ_I."""
 
+    def __eq__(self, other):
+        """Returns ``True`` if two domains are equivalent. """
+        if isinstance(other, GaussianRationalField):
+            return True
+        else:
+            return NotImplemented
+
+    def __hash__(self):
+        """Compute hash code of ``self``. """
+        return hash('QQ_I')
+
+    @property
+    def has_CharacteristicZero(self):
+        return True
+
+    def characteristic(self):
+        return 0
+
     def get_ring(self):
         """Returns a ring associated with ``self``. """
         return ZZ_I
@@ -627,4 +678,9 @@ class GaussianRationalField(GaussianDomain, Field):
         """Convert a QQ_I element to QQ_I."""
         return a
 
+    def from_ComplexField(K1, a, K0):
+        """Convert a ComplexField element to QQ_I."""
+        return K1.new(QQ.convert(a.real), QQ.convert(a.imag))
+
+
 QQ_I = GaussianRational._parent = GaussianRationalField()
